#ifndef _HT3035_H
#define _HT3035_H

#define HT3035_BASE (0x280)
#define HT3035_REG(offset) ((offset) + HT3035_BASE)

/* Command Register */
#define HT3035_WR0L	HT3035_REG(0x00)
#define 	HT3035_WR0L_SET_R	(0x00) /* Range setting */
#define 	HT3035_WR0L_SET_K	(0x01) /* Jerk setting */
#define 	HT3035_WR0L_SET_A	(0x02) /* Acceleration setting */
#define 	HT3035_WR0L_SET_D	(0x03) /* Deceleration setting */
#define 	HT3035_WR0L_SET_SV	(0x04) /* Initial velocity setting */
#define 	HT3035_WR0L_SET_V	(0x05) /* Drive velocity setting */
#define 	HT3035_WR0L_SET_P	(0x06) /* Output pulse numbers */
#define 	HT3035_WR0L_SET_DP	(0x07) /* Manual deceleration point setting */
#define 	HT3035_WR0L_SET_LP	(0x09) /* Logical position counter setting */
#define 	HT3035_WR0L_SET_EP	(0x0a) /* Real position counter setting */
#define 	HT3035_WR0L_SET_CP	(0x0b) /* COMP + register setting */
#define 	HT3035_WR0L_SET_CM	(0x0c) /* COMP - register setting */
#define 	HT3035_WR0L_SET_AO	(0x0d) /* Acceleration counter offsetting */
#define 	HT3035_WR0L_SET_NOP	(0x0f) /* NOP (For axis switching) */
#define 	HT3035_WR0L_GET_LP	(0x10) /* Logical position counter reading */
#define 	HT3035_WR0L_GET_EP	(0x11) /* Real position counter reading */
#define 	HT3035_WR0L_GET_CV	(0x12) /* Current drive velocity reading */
#define 	HT3035_WR0L_GET_CA	(0x13) /* Acceleration / deceleration reading */
#define 	HT3035_WR0L_DRV_PF	(0x20) /* + direction fixed driving */
#define 	HT3035_WR0L_DRV_MF	(0x21) /* - direction fixed driving */
#define 	HT3035_WR0L_DRV_PC	(0x22) /* + direction continuous driving */
#define 	HT3035_WR0L_DRV_MC	(0x23) /* - direction continuous driving */
#define 	HT3035_WR0L_DRV_H	(0x24) /* Drive start holding */
#define 	HT3035_WR0L_DRV_HR	(0x25) /* Drive start holding release/stop status clear */
#define 	HT3035_WR0L_DRV_DS	(0x26) /* Decelerating stop */
#define 	HT3035_WR0L_DRV_SS	(0x27) /* Sudden stop */
#define 	HT3035_WR0L_SET_HM	(0x60) /* Automatic home search mode setting */
#define 	HT3035_WR0L_SET_HV	(0x61) /* Home search velocity setting */
#define 	HT3035_WR0L_HS		(0x62) /* Automatic home search execution */
#define		HT3035_WR0L_CC		(0x63) /* Deviation counter clearing output */
#define HT3035_WR0H	HT3035_REG(0x01)
#define 	HT3035_WR0H_RESET	(1<<7)
#define 	HT3035_WR0H_X_AXIS	(1<<0)
#define 	HT3035_WR0H_Y_AXIS	(1<<1)
#define 	HT3035_WR0H_Z_AXIS	(1<<2)
#define 	HT3035_WR0H_U_AXIS	(1<<3)
#define 	HT3035_WR0H_ALL_AXIS	(0x0f)

/* Mode Register1 */
#define HT3035_WR1	HT3035_REG(0x02)
#define 	HT3035_WR1_SP0_L	(1<<0)
#define 	HT3035_WR1_SP0_E	(1<<1)
#define 	HT3035_WR1_SP1_L	(1<<2)
#define 	HT3035_WR1_SP1_E	(1<<3)
#define 	HT3035_WR1_SP2_L	(1<<4)
#define 	HT3035_WR1_SP2_E	(1<<5)
#define 	HT3035_WR1_EPCLR	(1<<6)
#define 	HT3035_WR1_EPINV	(1<<7)
#define 	HT3035_WR1_SMOD		(1<<8)
#define 	HT3035_WR1_P_GE_CM	(1<<9)
#define 	HT3035_WR1_P_LT_CM	(1<<10)
#define 	HT3035_WR1_P_LT_CP	(1<<11)
#define 	HT3035_WR1_P_GE_CP	(1<<12)
#define 	HT3035_WR1_C_END	(1<<13)
#define		HT3035_WR1_C_STA	(1<<14)
#define 	HT3035_WR1_D_END	(1<<15)

/* Mode Register2 */
#define HT3035_WR2	HT3035_REG(0x04)
#define 	HT3035_WR2_SLMTP	(1<<0)
#define 	HT3035_WR2_SLMTM	(1<<1)
#define 	HT3035_WR2_LMTMD	(1<<2)
#define 	HT3035_WR2_HLMTP	(1<<3)
#define 	HT3035_WR2_HLMTM	(1<<4)
#define 	HT3035_WR2_CMPSL	(1<<5)
#define 	HT3035_WR2_PLSMD	(1<<6)
#define 	HT3035_WR2_PLS_L	(1<<7)
#define 	HT3035_WR2_DIR_L	(1<<8)
#define 	HT3035_WR2_PINMD	(1<<9)
#define 	HT3035_WR2_PIND0	(1<<10)
#define 	HT3035_WR2_PIND1	(1<<11)
#define 	HT3035_WR2_ALM_L	(1<<12)
#define 	HT3035_WR2_ALM_E	(1<<13)
#define 	HT3035_WR2_INP_L	(1<<14)
#define 	HT3035_WR2_INP_E	(1<<15)

/* Mode Register3 */
#define HT3035_WR3	HT3035_REG(0x06)
#define 	HT3035_WR3_MANLD	(1<<0)
#define 	HT3035_WR3_DSNDE	(1<<1)
#define 	HT3035_WR3_SACC		(1<<2)
#define 	HT3035_WR3_EXOP0	(1<<3)
#define 	HT3035_WR3_EXOP1	(1<<4)
#define 	HT3035_WR3_AVTRI	(1<<5)
#define 	HT3035_WR3_VRING	(1<<6)
#define 	HT3035_WR3_FE0		(1<<8)
#define 	HT3035_WR3_FE1		(1<<9)
#define 	HT3035_WR3_FE2		(1<<10)
#define 	HT3035_WR3_FE3		(1<<11)
#define 	HT3035_WR3_FL0		(1<<13)
#define 	HT3035_WR3_FL1		(1<<14)
#define 	HT3035_WR3_FL2		(1<<15)

/* Output Register */
#define HT3035_WR4	HT3035_REG(0x08)
#define 	HT3035_WR4_XOUT0	(1<<0)
#define 	HT3035_WR4_XOUT1	(1<<1)
#define 	HT3035_WR4_XOUT2	(1<<2)
#define 	HT3035_WR4_XOUT3	(1<<3)
#define 	HT3035_WR4_YOUT0	(1<<4)
#define 	HT3035_WR4_YOUT1	(1<<5)
#define 	HT3035_WR4_YOUT2	(1<<6)
#define 	HT3035_WR4_YOUT3	(1<<7)
#define 	HT3035_WR4_ZOUT0	(1<<8)
#define 	HT3035_WR4_ZOUT1	(1<<9)
#define 	HT3035_WR4_ZOUT2	(1<<10)
#define 	HT3035_WR4_ZOUT3	(1<<11)
#define 	HT3035_WR4_UOUT0	(1<<12)
#define 	HT3035_WR4_UOUT1	(1<<13)
#define 	HT3035_WR4_UOUT2	(1<<14)
#define 	HT3035_WR4_UOUT3	(1<<15)

#define HT3035_WR5	HT3035_REG(0x0a)
#define 	HT3035_WR5_XOT0E	(1<<0)
#define 	HT3035_WR5_XOT1E	(1<<1)
#define 	HT3035_WR5_XOT2E	(1<<2)
#define 	HT3035_WR5_XOT3E	(1<<3)
#define 	HT3035_WR5_YOT0E	(1<<4)
#define 	HT3035_WR5_YOT1E	(1<<5)
#define 	HT3035_WR5_YOT2E	(1<<6)
#define 	HT3035_WR5_YOT3E	(1<<7)
#define 	HT3035_WR5_ZOT0E	(1<<8)
#define 	HT3035_WR5_ZOT1E	(1<<9)
#define 	HT3035_WR5_ZOT2E	(1<<10)
#define 	HT3035_WR5_ZOT3E	(1<<11)
#define 	HT3035_WR5_UOT0E	(1<<12)
#define 	HT3035_WR5_UOT1E	(1<<13)
#define 	HT3035_WR5_UOT2E	(1<<14)
#define 	HT3035_WR5_UOT3E	(1<<15)

/* Data-Write Register */
#define HT3035_WR6	HT3035_REG(0x0c)
#define HT3035_WR7	HT3035_REG(0x0e)

/* Main Status Register */
#define HT3035_RR0	HT3035_REG(0x00)
#define 	HT3035_RR0_X_DRV	(1<<0)
#define 	HT3035_RR0_Y_DRV	(1<<1)
#define 	HT3035_RR0_Z_DRV	(1<<2)
#define 	HT3035_RR0_U_DRV	(1<<3)
#define 	HT3035_RR0_X_ERR	(1<<4)
#define 	HT3035_RR0_Y_ERR	(1<<5)
#define 	HT3035_RR0_Z_ERR	(1<<6)
#define 	HT3035_RR0_U_ERR	(1<<7)
#define 	HT3035_RR0_X_HOM	(1<<8)
#define 	HT3035_RR0_Y_HOM	(1<<9)
#define 	HT3035_RR0_Z_HOM	(1<<10)
#define 	HT3035_RR0_U_HOM	(1<<11)

/* Status Register1 */
#define HT3035_RR1	HT3035_REG(0x02)
#define 	HT3035_RR1_CMPP		(1<<0)
#define 	HT3035_RR1_CMPM		(1<<1)
#define 	HT3035_RR1_ASND		(1<<2)
#define 	HT3035_RR1_CNST		(1<<3)
#define 	HT3035_RR1_DSND		(1<<4)
#define 	HT3035_RR1_AASND	(1<<5)
#define 	HT3035_RR1_ACNST	(1<<6)
#define 	HT3035_RR1_ADSND	(1<<7)
#define 	HT3035_RR1_STOP0	(1<<8)
#define 	HT3035_RR1_STOP1	(1<<9)
#define 	HT3035_RR1_STOP2	(1<<10)
#define 	HT3035_RR1_LMTP		(1<<12)
#define 	HT3035_RR1_LMTM		(1<<13)
#define 	HT3035_RR1_ALARM	(1<<14)
#define 	HT3035_RR1_EMG		(1<<15)

/* Status Register2 */
#define HT3035_RR2	HT3035_REG(0x04)
#define 	HT3035_RR2_ST_SLMTP	(1<<0)
#define 	HT3035_RR2_ST_SLMTM	(1<<1)
#define 	HT3035_RR2_ST_HLMTP	(1<<2)
#define 	HT3035_RR2_ST_HLMTM	(1<<3)
#define 	HT3035_RR2_ST_ALARM	(1<<4)
#define 	HT3035_RR2_ST_EMG	(1<<5)
#define 	HT3035_RR2_ST_HOME	(1<<7)
#define 	HT3035_RR2_ST_HMST0	(1<<8)
#define 	HT3035_RR2_ST_HMST1	(1<<9)
#define 	HT3035_RR2_ST_HMST2	(1<<10)
#define 	HT3035_RR2_ST_HMST3	(1<<11)
#define 	HT3035_RR2_ST_HMST4	(1<<12)

/* Status Register3 */
#define HT3035_RR3	HT3035_REG(0x06)
#define 	HT3035_RR3_ST_P_GE_CM	(1<<1)
#define 	HT3035_RR3_ST_P_LT_CM	(1<<2)
#define 	HT3035_RR3_ST_P_LT_CP	(1<<3)
#define 	HT3035_RR3_ST_P_GE_CP	(1<<4)
#define 	HT3035_RR3_ST_C_END	(1<<5)
#define		HT3035_RR3_ST_C_STA	(1<<6)
#define 	HT3035_RR3_ST_D_END	(1<<7)

/* Input Register */
#define HT3035_RR4	HT3035_REG(0x08)
#define 	HT3035_RR4_X_ST0	(1<<0)
#define 	HT3035_RR4_X_ST1	(1<<1)
#define 	HT3035_RR4_X_ST2	(1<<2)
#define 	HT3035_RR4_EMGN		(1<<3)
#define 	HT3035_RR4_X_EXP	(1<<4)
#define 	HT3035_RR4_X_EXM	(1<<5)
#define 	HT3035_RR4_X_INP	(1<<6)
#define 	HT3035_RR4_X_ALM	(1<<7)
#define 	HT3035_RR4_Y_ST0	(1<<8)
#define 	HT3035_RR4_Y_ST1	(1<<9)
#define 	HT3035_RR4_Y_ST2	(1<<10)
#define 	HT3035_RR4_Y_EXP	(1<<12)
#define 	HT3035_RR4_Y_EXM	(1<<13)
#define 	HT3035_RR4_Y_INP	(1<<14)
#define 	HT3035_RR4_Y_ALM	(1<<15)
#define HT3035_RR5	HT3035_REG(0x0a)
#define 	HT3035_RR5_Z_ST0	(1<<0)
#define 	HT3035_RR5_Z_ST1	(1<<1)
#define 	HT3035_RR5_Z_ST2	(1<<2)
#define 	HT3035_RR5_Z_EXP	(1<<4)
#define 	HT3035_RR5_Z_EXM	(1<<5)
#define 	HT3035_RR5_Z_INP	(1<<6)
#define 	HT3035_RR5_Z_ALM	(1<<7)
#define 	HT3035_RR5_U_ST0	(1<<8)
#define 	HT3035_RR5_U_ST1	(1<<9)
#define 	HT3035_RR5_U_ST2	(1<<10)
#define 	HT3035_RR5_U_EXP	(1<<12)
#define 	HT3035_RR5_U_EXM	(1<<13)
#define 	HT3035_RR5_U_INP	(1<<14)
#define 	HT3035_RR5_U_ALM	(1<<15)

/* Data-Read Register */
#define HT3035_RR6	HT3035_REG(0x0c)
#define HT3035_RR7	HT3035_REG(0x0e)

/* Command Data Range */
#define R_MIN  (16000)
#define R_MAX  (8000000)
#define K_MIN  (0x1)
#define K_MAX  (0xffff)
#define A_MIN  (1)
#define A_MAX  (8000)
#define D_MIN  (A_MIN)
#define D_MAX  (A_MAX)
#define SV_MIN (1)
#define SV_MAX (8000)
#define V_MIN  (1)
#define V_MAX  (8000)
#define P_MIN  (0x0)
#define P_MAX  (0xfffffff)
#define DP_MIN (0x0)
#define DP_MAX (0xfffffff)
#define LP_MIN (-0x8000000)
#define LP_MAX (0x7ffffff)
#define EP_MIN (-0x8000000)
#define EP_MAX (0x7ffffff)
#define CP_MIN (-0x8000000)
#define CP_MAX (0x7ffffff)
#define CM_MIN (-0x8000000)
#define CM_MAX (0x7ffffff)
#define AO_MIN (-0x8000)
#define AO_MAX (0x7fff)
#define HV_MIN (1)
#define HV_MAX (8000)

/* Command Parameter Range */
#define MULTIPLE_MIN (8000000/R_MAX)
#define MULTIPLE_MAX (8000000/R_MIN)
#define ACCELERATION_MIN  (A_MIN*125*MULTIPLE_MIN)
#define ACCELERATION_MAX  (A_MAX*125*MULTIPLE_MAX)
#define DECELERATION_MIN  (D_MIN*125*MULTIPLE_MIN)
#define DECELERATION_MAX  (D_MAX*125*MULTIPLE_MAX)
#define VELOCITY_MIN (V_MIN*MULTIPLE_MIN)
#define VELOCITY_MAX (V_MAX*MULTIPLE_MAX)
#define JERK_MIN (125*MULTIPLE_MIN/2/K_MAX)
#define JERK_MAX (125*MULTIPLE_MAX/2/K_MIN)
#define INIT_VELOCITY_MIN (SV_MIN*MULTIPLE_MIN)
#define INIT_VELOCITY_MAX (SV_MAX*MULTIPLE_MAX)
#define NR_PULSE_MIN (P_MIN)
#define NR_PULSE_MAX (P_MAX)

#define	DIV_ROUND_UP(x,y) (((x) + ((y) - 1)) / (y))

/* Data Conversion Macros */
#define MULTIPLE_TO_R(val) (8000000/val)
#define ACCELERATION_TO_A(val,mul) (DIV_ROUND_UP(val/125,mul))
#define DECELERATION_TO_D(val,mul) (DIV_ROUND_UP(val/125,mul))
#define VELOCITY_TO_V(val,mul) (DIV_ROUND_UP(val,mul))
#define INIT_VELOCITY_TO_SV(val,mul) (DIV_ROUND_UP(val,mul))
#define NR_PULSE_TO_P(val) (val)

struct drive_param {
	uint32_t multiple;
	uint32_t velocity;
	uint32_t initial_velocity;
	uint32_t acceleration;
	uint32_t deceleration;
	uint32_t jerk;
};

struct drive_info {
	unsigned char axes;
	int curve;
	struct drive_param arg; // argument value
	struct drive_param reg; // register value
	uint32_t nr_pulse;
};

#define CURVE_I (1<<0)
#define CURVE_V (1<<1)
#define CURVE_A (1<<2)
#define CURVE_S (1<<3)

#define	CURVE_CONSTANT	(CURVE_I | CURVE_V)
#define CURVE_LINEAR	(CURVE_CONSTANT | CURVE_A)
#define CURVE_PERFECT_S	(CURVE_LINEAR | CURVE_S)

#endif
